@@ -2429,13 +2429,13 @@ clearlooks_draw_arrow (cairo_t *cr,
 
 	if (widget->disabled)
 	{
+                cairo_save (cr);
 		_clearlooks_draw_arrow (cr, &colors->shade[0],
 		                        arrow->direction, arrow->type,
 		                        tx+0.5, ty+0.5, width, height);
+                cairo_restore (cr);
 	}
 
-	cairo_identity_matrix (cr);
-
 	_clearlooks_draw_arrow (cr, color, arrow->direction, arrow->type,
 	                        tx, ty, width, height);
 }
