@@ -1,24 +1,25 @@
-/* Glide theme engine
- * Copyright (C) 2006 Andrew Johnson
+/* Glide - a cairo based GTK+ engine
+ * Copyright (C) 2006 Andrew Johnson <acjgenius@earthlink.net>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Project contact: <gnome-themes-list@gnome.org>
  *
- * Written by Andrew Johnson <acjgenius@earthlink.net>
- */ 
- 
+ */
+
+
 #include "glide_gtk2_engine.h"
 #include "glide_gtk2_support.h"
 #include "glide_gtk2_drawing.h"
@@ -151,9 +152,10 @@ glide_style_realize (GtkStyle * style)
 		glide_style->bg_solid[i] = ge_cairo_color_pattern(&base);
 
 		glide_style->bg_image[i] = NULL;
-		if ((style->bg_pixmap[i]) && (style->bg_pixmap[i] != (GdkPixmap*) GDK_PARENT_RELATIVE))
+                if (GTK_STYLE (style)->rc_style->bg_pixmap_name[i] && 
+                    GTK_STYLE (style)->background[i] != NULL)
 		{
-			glide_style->bg_image[i] = ge_cairo_pixmap_pattern(style->bg_pixmap[i]);
+			glide_style->bg_image[i] = ge_cairo_pattern_pattern(style->background[i]);
 		}
 
 		glide_style->bg_gradient[0][i] = ge_cairo_linear_shade_gradient_pattern(&base, 1.05, 0.95, FALSE);
