@@ -38,10 +38,9 @@
 /* Standard Border Function */
 void
 hc_draw_shadow(GtkStyle * style,
-	       GdkWindow * window,
+	       cairo_t * cr,
 	       GtkStateType state_type,
 	       GtkShadowType shadow_type,
-	       GdkRectangle * area,
 	       GtkWidget * widget,
 	       const gchar * detail,
 	       gint x,
@@ -55,13 +54,10 @@ hc_draw_shadow(GtkStyle * style,
 	gint line_width;
 	gint clip_x = x, clip_y = y, clip_width = width, clip_height = height;
 
-	cairo_t *canvas;
-
 	/***********************************************/
 	/* GTK Sanity Checks                           */
 	/***********************************************/
 	CHECK_ARGS
-	SANITIZE_SIZE 
 
 
 	/***********************************************/
@@ -103,7 +99,7 @@ hc_draw_shadow(GtkStyle * style,
 		/* Force Border To Use Foreground Widget State */
 		if (widget)
 		{
-			foreground = HC_STYLE(style)->color_cube.fg[widget->state];
+			foreground = HC_STYLE(style)->color_cube.fg[gtk_widget_get_state(widget)];
 		}
 	}
 
@@ -111,7 +107,7 @@ hc_draw_shadow(GtkStyle * style,
 	/* Entry - Force Border To Use Foreground Matching Widget State */
 	if (CHECK_DETAIL(detail, "entry") && !ge_is_combo(widget))
 	{
-		foreground = HC_STYLE(style)->color_cube.fg[widget ? widget->state : GTK_STATE_NORMAL];
+		foreground = HC_STYLE(style)->color_cube.fg[widget ? gtk_widget_get_state(widget) : GTK_STATE_NORMAL];
 	}
 
 
@@ -131,10 +127,10 @@ hc_draw_shadow(GtkStyle * style,
 
 
 		/* Force Border To Use Foreground Matching Parent State */
-		if ((widget) && (widget->parent))
+		if ((widget) && (gtk_widget_get_parent(widget)))
 		{
-			gtk_widget_ensure_style(widget->parent);
-			ge_gdk_color_to_cairo(&widget->parent->style->fg[gtk_widget_get_state (widget)], &foreground);
+			gtk_widget_ensure_style(gtk_widget_get_parent(widget));
+			ge_gdk_color_to_cairo(&gtk_widget_get_style(gtk_widget_get_parent(widget))->fg[gtk_widget_get_state (widget)], &foreground);
 		}
 	}
 
@@ -142,22 +138,18 @@ hc_draw_shadow(GtkStyle * style,
 	/***********************************************/
 	/* Draw Border                                 */
 	/***********************************************/
-	canvas = ge_gdk_drawable_to_cairo (window, area);
-
 	/* Clip Border Too Passed Size */
-	cairo_rectangle(canvas, clip_x, clip_y, clip_width, clip_height);
-	cairo_clip(canvas);
+	cairo_rectangle(cr, clip_x, clip_y, clip_width, clip_height);
+	cairo_clip(cr);
 
 	/* Set Line Style */
-	ge_cairo_set_color(canvas, &foreground);
-	cairo_set_line_cap(canvas, CAIRO_LINE_CAP_BUTT);
+	ge_cairo_set_color(cr, &foreground);
+	cairo_set_line_cap(cr, CAIRO_LINE_CAP_BUTT);
 
-	cairo_set_line_width (canvas, line_width);
-	ge_cairo_inner_rectangle (canvas, x, y, width, height);
+	cairo_set_line_width (cr, line_width);
+	ge_cairo_inner_rectangle (cr, x, y, width, height);
 	
-	cairo_stroke(canvas);
-
-	cairo_destroy(canvas);
+	cairo_stroke(cr);
 }
 
 
@@ -166,10 +158,9 @@ hc_draw_shadow(GtkStyle * style,
  */
 void 
 hc_draw_shadow_gap (GtkStyle       *style,
-		    GdkWindow      *window,
+		    cairo_t        *cr,
 		    GtkStateType    state_type,
 		    GtkShadowType   shadow_type,
-		    GdkRectangle   *area,
 		    GtkWidget      *widget,
 		    const gchar    *detail,
 		    gint            x,
@@ -183,13 +174,11 @@ hc_draw_shadow_gap (GtkStyle       *style,
 	/* Border Uses Foreground Color */
 	CairoColor *foreground = &HC_STYLE(style)->color_cube.fg[state_type];
 	gint line_width;
-	cairo_t *canvas;
 
 	/***********************************************/
 	/* GTK Sanity Checks                           */
 	/***********************************************/
 	CHECK_ARGS
-	SANITIZE_SIZE 
 
 	if (shadow_type == GTK_SHADOW_NONE)
 		return;
@@ -208,32 +197,27 @@ hc_draw_shadow_gap (GtkStyle       *style,
 	/***********************************************/
 	/* Draw Border                                 */
 	/***********************************************/
-	canvas = ge_gdk_drawable_to_cairo (window, area);
-
 	/* Create And Clip Too Path To Ignore Gap */
-	hc_simple_border_gap_clip(canvas, line_width, x, y, width, height, gap_side, gap_pos, gap_size);
+	hc_simple_border_gap_clip(cr, line_width, x, y, width, height, gap_side, gap_pos, gap_size);
 
 
 	/* Set Line Style */
-	ge_cairo_set_color(canvas, foreground);
-	cairo_set_line_cap(canvas, CAIRO_LINE_CAP_BUTT);
+	ge_cairo_set_color(cr, foreground);
+	cairo_set_line_cap(cr, CAIRO_LINE_CAP_BUTT);
 
-	cairo_set_line_width (canvas, line_width);
-	ge_cairo_inner_rectangle (canvas, x, y, width, height);
+	cairo_set_line_width (cr, line_width);
+	ge_cairo_inner_rectangle (cr, x, y, width, height);
 	
-	cairo_stroke(canvas);
-
-	cairo_destroy(canvas);
+	cairo_stroke(cr);
 }
 
 
 /* Border Function For Notebooks Tabs */
 void 
 hc_draw_extension (GtkStyle       *style,
-		   GdkWindow      *window,
+		   cairo_t        *cr,
 		   GtkStateType    state_type,
 		   GtkShadowType   shadow_type,
-		   GdkRectangle   *area,
 		   GtkWidget      *widget,
 		   const gchar    *detail,
 		   gint            x,
@@ -253,13 +237,10 @@ hc_draw_extension (GtkStyle       *style,
 	gint widget_x = 0, widget_y = 0, widget_width = 0, widget_height = 0;
 	gint clip_x = x, clip_y = y, clip_width = width, clip_height = height;
 
-	cairo_t *canvas;
-
 	/***********************************************/
 	/* GTK Sanity Checks                           */
 	/***********************************************/
 	CHECK_ARGS
-	SANITIZE_SIZE 
 
 
 	/***********************************************/
@@ -281,10 +262,12 @@ hc_draw_extension (GtkStyle       *style,
 	 */
 	if (widget && (GE_IS_NOTEBOOK (widget)))
 	{
-		widget_x = (widget->allocation.x + GTK_CONTAINER (widget)->border_width);
-		widget_y = (widget->allocation.y + GTK_CONTAINER (widget)->border_width);
-		widget_width = (widget->allocation.width - 2*GTK_CONTAINER (widget)->border_width);
-		widget_height = (widget->allocation.height - 2*GTK_CONTAINER (widget)->border_width);
+		GtkAllocation allocation;
+		gtk_widget_get_allocation (widget, &allocation);
+		widget_x = (allocation.x + gtk_container_get_border_width (GTK_CONTAINER (widget)));
+		widget_y = (allocation.y + gtk_container_get_border_width (GTK_CONTAINER (widget)));
+		widget_width = (allocation.width - 2*gtk_container_get_border_width (GTK_CONTAINER (widget)));
+		widget_height = (allocation.height - 2*gtk_container_get_border_width (GTK_CONTAINER (widget)));
 	}
 
 	switch (gap_side)
@@ -339,31 +322,28 @@ hc_draw_extension (GtkStyle       *style,
 	/***********************************************/
 	/* Draw Border                                 */
 	/***********************************************/
-	canvas = ge_gdk_drawable_to_cairo (window, area);
 
 	/* Clip Too Size */
-	cairo_rectangle(canvas, clip_x, clip_y, clip_width, clip_height);
-	cairo_clip(canvas);
+	cairo_rectangle(cr, clip_x, clip_y, clip_width, clip_height);
+	cairo_clip(cr);
 
 
 	/* Set Fill Style */
-	ge_cairo_set_color(canvas, background);
+	ge_cairo_set_color(cr, background);
 
 	/* Fill Rectangle */
-	cairo_rectangle (canvas, x, y, width, height);
-	cairo_fill(canvas);
+	cairo_rectangle (cr, x, y, width, height);
+	cairo_fill(cr);
 
 
 	/* Set Line Style */
-	ge_cairo_set_color(canvas, foreground);
-	cairo_set_line_cap(canvas, CAIRO_LINE_CAP_BUTT);
+	ge_cairo_set_color(cr, foreground);
+	cairo_set_line_cap(cr, CAIRO_LINE_CAP_BUTT);
 
-	cairo_set_line_width (canvas, line_width);
-	ge_cairo_inner_rectangle (canvas, x, y, width, height);
+	cairo_set_line_width (cr, line_width);
+	ge_cairo_inner_rectangle (cr, x, y, width, height);
 	
-	cairo_stroke(canvas);
-
-	cairo_destroy(canvas);
+	cairo_stroke(cr);
 }
 
 
@@ -371,10 +351,9 @@ hc_draw_extension (GtkStyle       *style,
 		or With A Single Pixel Line */
 void
 hc_draw_flat_box (GtkStyle	*style,
-                  GdkWindow	*window,
+                  cairo_t  	*cr,
                   GtkStateType	 state_type,
                   GtkShadowType	 shadow_type,
-                  GdkRectangle	*area,
                   GtkWidget	*widget,
                   const gchar	*detail,
                   gint		 x,
@@ -384,14 +363,14 @@ hc_draw_flat_box (GtkStyle	*style,
 {
 	if (detail && !strcmp ("tooltip", detail))
 	{
-		hc_draw_box (style, window, state_type, shadow_type, area,
+		hc_draw_box (style, cr, state_type, shadow_type,
 				widget, detail, x, y, width, height);
 	}
 	else
 	{
 		GtkStyleClass *hc_parent_class;
 		hc_parent_class = GTK_STYLE_CLASS (g_type_class_peek_parent (G_OBJECT_GET_CLASS(style)));
-		hc_parent_class->draw_flat_box (style, window, state_type, shadow_type, area,
+		hc_parent_class->draw_flat_box (style, cr, state_type, shadow_type,
 							widget, detail, x, y, width, height);
 	}
 }
@@ -401,10 +380,9 @@ hc_draw_flat_box (GtkStyle	*style,
 	Ensures Fill And Border */
 void
 hc_draw_box (GtkStyle	*style,
-                  GdkWindow	*window,
+                  cairo_t  	*cr,
                   GtkStateType	 state_type,
                   GtkShadowType	 shadow_type,
-                  GdkRectangle	*area,
                   GtkWidget	*widget,
                   const gchar	*detail,
                   gint		 x,
@@ -416,7 +394,6 @@ hc_draw_box (GtkStyle	*style,
 	/* GTK Sanity Checks                           */
 	/***********************************************/
 	CHECK_ARGS
-	SANITIZE_SIZE 
 
 
 	/***********************************************/
@@ -433,15 +410,15 @@ hc_draw_box (GtkStyle	*style,
 	/***********************************************/
 	/* Draw Fill                                   */
 	/***********************************************/
-	gtk_style_apply_default_background (style, window,
-        					widget && !gtk_widget_get_has_window (widget),
-						state_type, area, x, y, width, height);
+	gtk_style_apply_default_background (style, cr,
+        				    gtk_widget_get_window (widget),
+					    state_type, x, y, width, height);
   
 
 	/***********************************************/
 	/* Draw Border                                 */
 	/***********************************************/
-	hc_draw_shadow (style, window, state_type, shadow_type, area, widget, detail, 
+	hc_draw_shadow (style, cr, state_type, shadow_type, widget, detail, 
 					x, y, width, height);    
 }
 
@@ -453,10 +430,9 @@ hc_draw_box (GtkStyle	*style,
  */
 void 
 hc_draw_box_gap (GtkStyle       *style,
-		 GdkWindow      *window,
+		 cairo_t        *cr,
 		 GtkStateType    state_type,
 		 GtkShadowType   shadow_type,
-		 GdkRectangle   *area,
 		 GtkWidget      *widget,
 		 const gchar    *detail,
 		 gint            x,
@@ -471,21 +447,20 @@ hc_draw_box_gap (GtkStyle       *style,
 	/* GTK Sanity Checks                           */
 	/***********************************************/
 	CHECK_ARGS
-	SANITIZE_SIZE 
 
 
 	/***********************************************/
 	/* Draw Fill                                   */
 	/***********************************************/
-	gtk_style_apply_default_background (style, window,
-	                                    widget && !gtk_widget_get_has_window (widget),
-	                                    state_type, area, x, y, width, height);
+	gtk_style_apply_default_background (style, cr,
+	                                    gtk_widget_get_window (widget),
+	                                    state_type, x, y, width, height);
 
 
 	/***********************************************/
 	/* Draw Border                                 */
 	/***********************************************/
-	hc_draw_shadow_gap (style, window, state_type, shadow_type, area, widget, detail, 
+	hc_draw_shadow_gap (style, cr , state_type, shadow_type, widget, detail, 
 					x, y, width, height, gap_side, gap_pos, gap_size);    
 }
 
@@ -498,10 +473,9 @@ hc_draw_box_gap (GtkStyle       *style,
  */
 void 
 hc_draw_handle (GtkStyle      *style,
-		GdkWindow     *window,
+		cairo_t       *cr,
 		GtkStateType   state_type,
 		GtkShadowType  shadow_type,
-		GdkRectangle  *area,
 		GtkWidget     *widget,
 		const gchar   *detail,
 		gint           x,
@@ -516,13 +490,11 @@ hc_draw_handle (GtkStyle      *style,
 	HcStyle *hc_style;
 	gdouble xx, yy;
 	CairoColor *light, *dark;
-	cairo_t *canvas;
 
 	/***********************************************/
 	/* GTK Sanity Checks                           */
 	/***********************************************/
 	CHECK_ARGS
-	SANITIZE_SIZE 
 
 
 	/***********************************************/
@@ -547,7 +519,7 @@ hc_draw_handle (GtkStyle      *style,
 	/***********************************************/
 	/* Draw Box                                    */
 	/***********************************************/
-	hc_draw_box (style, window, state_type, shadow_type, area, widget, 
+	hc_draw_box (style, cr, state_type, shadow_type, widget, 
 			detail, x, y, width, height);
 
 
@@ -559,24 +531,22 @@ hc_draw_handle (GtkStyle      *style,
 	light = &hc_style->color_cube.light[state_type];
 	dark = &hc_style->color_cube.dark[state_type];
 
-	canvas = ge_gdk_drawable_to_cairo (window, area);
-
 	/* Clip Too Size */
-	cairo_rectangle(canvas, clip_x, clip_y, clip_width, clip_height);
-	cairo_clip(canvas);
+	cairo_rectangle(cr, clip_x, clip_y, clip_width, clip_height);
+	cairo_clip(cr);
 
 	if (CHECK_DETAIL (detail, "paned"))
 	{
 		if (orientation == GTK_ORIENTATION_HORIZONTAL)
 		{
 			for (xx = x + width/2.0 - 15; xx <= x + width/2.0 + 15; xx += 5)
-				do_hc_draw_dot (canvas, light, dark, xx, y + height/2.0);
+				do_hc_draw_dot (cr, light, dark, xx, y + height/2.0);
 		}
 		else
 		{
 			for (yy = y + height/2 - 15; yy <= y + height/2.0 + 15; yy += 5)
 			{
-				do_hc_draw_dot (canvas, light, dark, x + width/2.0, yy);
+				do_hc_draw_dot (cr, light, dark, x + width/2.0, yy);
 			}
 		}
 	}
@@ -585,25 +555,22 @@ hc_draw_handle (GtkStyle      *style,
 		if (orientation == GTK_ORIENTATION_HORIZONTAL)
 		{
 			for (xx = x + xthick + (width/2 - xthick) % 5; xx <= x + width - xthick*2; xx += 5)
-				do_hc_draw_dot (canvas, light, dark, xx + 2, y + height/2);
+				do_hc_draw_dot (cr, light, dark, xx + 2, y + height/2);
 		}
 		else
 		{
 			for (yy = y + ythick + (height/2 - ythick) % 5; yy <= y + height - ythick*2; yy += 5)
-				do_hc_draw_dot (canvas, light, dark, x + width/2, yy + 2);
+				do_hc_draw_dot (cr, light, dark, x + width/2, yy + 2);
 		}
 	}
-
-	cairo_destroy(canvas);
 }
 
 
 void
 hc_draw_slider (GtkStyle * style,
-	     GdkWindow * window,
+	     cairo_t * cr,
 	     GtkStateType state_type,
 	     GtkShadowType shadow_type,
-	     GdkRectangle * area,
 	     GtkWidget * widget,
 	     const gchar * detail,
 	     gint x,
@@ -612,27 +579,23 @@ hc_draw_slider (GtkStyle * style,
              gint height, 
              GtkOrientation orientation)
 {
-	cairo_t *canvas;
 	gint line_width;
 
 	CHECK_ARGS
-	SANITIZE_SIZE
  
-	canvas = ge_gdk_drawable_to_cairo (window, area);
-
 	/***********************************************/
 	/* Draw Box                                    */
 	/***********************************************/
 	line_width = HC_STYLE(style)->edge_thickness;
 
-	hc_draw_box (style, window, state_type, shadow_type, area, widget, 
+	hc_draw_box (style, cr, state_type, shadow_type, widget, 
 			detail, x, y, width, height);
 
 	if (GE_IS_SCALE(widget))
 	{
 		if (orientation == GTK_ORIENTATION_HORIZONTAL)
 		{
-			do_hc_draw_line (canvas, &HC_STYLE(style)->color_cube.fg[state_type],
+			do_hc_draw_line (cr, &HC_STYLE(style)->color_cube.fg[state_type],
 						line_width /2, 
 						x + ceil(width/2.0) + 0.5, 
 						y + line_width, 
@@ -641,7 +604,7 @@ hc_draw_slider (GtkStyle * style,
 		}
 		else
 		{
-			do_hc_draw_line (canvas, &HC_STYLE(style)->color_cube.fg[state_type],
+			do_hc_draw_line (cr, &HC_STYLE(style)->color_cube.fg[state_type],
 						line_width /2, 
 						x + line_width, 
 						y + ceil(height/2.0) + 0.5,
@@ -649,17 +612,14 @@ hc_draw_slider (GtkStyle * style,
 						y + ceil(height/2.0) + 0.5);     
 		}
 	}
-
-	cairo_destroy(canvas);
 }
 
 /* Draw Check Buttons Check & Border */
 void 
 hc_draw_check (GtkStyle      *style,
-	       GdkWindow     *window,
+	       cairo_t       *cr,
 	       GtkStateType   state_type,
 	       GtkShadowType  shadow_type,
-	       GdkRectangle  *area,
 	       GtkWidget     *widget,
 	       const gchar   *detail,
 	       gint           x,
@@ -670,10 +630,8 @@ hc_draw_check (GtkStyle      *style,
 	HcStyle *hc_style = HC_STYLE (style);
 	gboolean inconsistent;
 	gint line_width;
-	cairo_t *cr;
 
 	CHECK_ARGS
-	SANITIZE_SIZE
 
 	/* Bug #351764 */
 	if ((!GTK_CHECK_VERSION(2,12,0)) && 
@@ -688,7 +646,6 @@ hc_draw_check (GtkStyle      *style,
 	inconsistent = (shadow_type == GTK_SHADOW_ETCHED_IN);
 
 	line_width = ceil(HC_STYLE(style)->edge_thickness/2.0);
-	cr = ge_gdk_drawable_to_cairo (window, area);
 
 	cairo_save(cr);
 
@@ -744,17 +701,14 @@ hc_draw_check (GtkStyle      *style,
 
 		cairo_restore (cr);
 	}
-
-	cairo_destroy(cr);
 }
 
 /* Draw Radio Button AKA Option Button Check & Border */
 void 
 hc_draw_option (GtkStyle      *style,
-		GdkWindow     *window,
+		cairo_t       *cr,
 		GtkStateType   state_type,
 		GtkShadowType  shadow_type,
-		GdkRectangle  *area,
 		GtkWidget     *widget,
 		const gchar   *detail,
 		gint           x,
@@ -763,7 +717,6 @@ hc_draw_option (GtkStyle      *style,
 		gint           height)
 {
 	HcStyle *hc_style = HC_STYLE (style);
-	cairo_t *cr;
 
 	gint centerX;
 	gint centerY;
@@ -771,7 +724,6 @@ hc_draw_option (GtkStyle      *style,
 	gboolean inconsistent;
 
 	CHECK_ARGS
-	SANITIZE_SIZE
 
 	/* Bug #351764 */
 	if ((!GTK_CHECK_VERSION(2,12,0)) && 
@@ -783,8 +735,6 @@ hc_draw_option (GtkStyle      *style,
 		height = HC_STYLE(style)->cell_indicator_size;
 	}
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	centerX = x + floor(width/2);
 	centerY = y + floor(height/2);
 	radius = floor(MIN(width,height)/2.0);
@@ -826,16 +776,13 @@ hc_draw_option (GtkStyle      *style,
 
 		cairo_stroke (cr);
 	}
-
-	cairo_destroy(cr);
 }
 
 void
 hc_draw_tab (GtkStyle      *style,
-	     GdkWindow     *window,
+	     cairo_t       *cr,
 	     GtkStateType   state_type,
 	     GtkShadowType  shadow_type,
-	     GdkRectangle  *area,
 	     GtkWidget     *widget,
 	     const gchar   *detail,
 	     gint           x,
@@ -847,7 +794,6 @@ hc_draw_tab (GtkStyle      *style,
 	GtkBorder indicator_spacing;
   
 	HcStyle *hc_style = HC_STYLE (style);
-	cairo_t *cr;
 
 	ge_option_menu_get_props (widget, &indicator_size, &indicator_spacing);
   
@@ -864,49 +810,38 @@ hc_draw_tab (GtkStyle      *style,
 	width = indicator_size.width;
 	height = indicator_size.height;
  
-	cr = ge_gdk_drawable_to_cairo(window, area);
-
 	do_hc_draw_arrow (cr, &hc_style->color_cube.fg[state_type],
 				GTK_ARROW_DOWN,TRUE, x, y,
 				width, height);
-
-
-	cairo_destroy(cr);
 }
 
 void
 hc_draw_layout (GtkStyle        *style,
-		GdkWindow       *window,
+		cairo_t         *cr,
 		GtkStateType     state_type,
 		gboolean         use_text,
-		GdkRectangle    *area,
 		GtkWidget       *widget,
 		const gchar     *detail,
 		gint             x,
 		gint             y,
 		PangoLayout     *layout)
 {
-	GdkGC *gc;
-
 	CHECK_ARGS
 
-	gc = use_text ? style->text_gc[state_type] : style->fg_gc[state_type];
+	if (use_text)
+		gdk_cairo_set_source_color (cr, &style->text[state_type]);
+	else
+		gdk_cairo_set_source_color (cr, &style->fg[state_type]);
 
-	if (area)
-		gdk_gc_set_clip_rectangle (gc, area);
-
-	gdk_draw_layout (window, gc, x, y, layout);
-
-	if (area)
-		gdk_gc_set_clip_rectangle (gc, NULL);
+	ge_cairo_transform_for_layout (cr, layout, x, y);
+	pango_cairo_show_layout (cr, layout);
 }
 
 void
 hc_draw_arrow (GtkStyle      *style,
-	       GdkWindow     *window,
+	       cairo_t       *cr,
 	       GtkStateType   state,
 	       GtkShadowType  shadow,
-	       GdkRectangle  *area,
 	       GtkWidget     *widget,
 	       const gchar   *detail,
 	       GtkArrowType   arrow_type,
@@ -918,13 +853,11 @@ hc_draw_arrow (GtkStyle      *style,
 {
 	gint line_width;
 	HcStyle *hc_style;
-	cairo_t *cr;
 
 	/***********************************************/
 	/* GTK Sanity Checks                           */
 	/***********************************************/
 	CHECK_ARGS
-	SANITIZE_SIZE
 
 	/***********************************************/
 	/* GTK Arrow Special Cases - adjust Size/Offset*/
@@ -983,19 +916,15 @@ hc_draw_arrow (GtkStyle      *style,
 	/* Draw Arrow                                  */
 	/***********************************************/
 	hc_style = HC_STYLE (style);
-	cr = ge_gdk_drawable_to_cairo(window, area);
 
 	do_hc_draw_arrow (cr, &hc_style->color_cube.fg[state], arrow_type, TRUE,
 				x, y, width+1, height+1);
-
-	cairo_destroy(cr);
 }
 
 void
 hc_draw_hline (GtkStyle     *style,
-	       GdkWindow    *window,
+	       cairo_t      *cr,
 	       GtkStateType  state_type,
-	       GdkRectangle  *area,
 	       GtkWidget     *widget,
 	       const gchar   *detail,
 	       gint          x1,
@@ -1003,27 +932,21 @@ hc_draw_hline (GtkStyle     *style,
 	       gint          y)
 {
 	HcStyle *hc_style = HC_STYLE (style);
-	cairo_t *cr;
 	gint line_width;
 
 	CHECK_ARGS
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	line_width = style->ythickness/2;
 
 	do_hc_draw_line (cr, &hc_style->color_cube.fg[state_type], (CHECK_DETAIL(detail, "label"))?1:2*line_width - 1, 
 	                 x1 + line_width + 2, y + style->ythickness/2 + 0.5,
 	                 x2 - line_width - 1, y + style->ythickness/2 + 0.5);
-
-	cairo_destroy(cr);
 }
 
 void
 hc_draw_vline (GtkStyle     *style,
-	       GdkWindow    *window,
+	       cairo_t      *cr,
 	       GtkStateType  state_type,
-	       GdkRectangle  *area,
 	       GtkWidget     *widget,
 	       const gchar   *detail,
 	       gint          y1,
@@ -1031,27 +954,21 @@ hc_draw_vline (GtkStyle     *style,
 	       gint          x)
 {
 	HcStyle *hc_style = HC_STYLE (style);
-	cairo_t *cr;
 	gint line_width;
 
 	CHECK_ARGS
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	line_width = style->xthickness/2;
 
 	do_hc_draw_line (cr, &hc_style->color_cube.fg[state_type],  (CHECK_DETAIL(detail, "label"))?1:2*line_width - 1, 
 	                 x + style->xthickness/2 + 0.5, y1,
 	                 x + style->xthickness/2 + 0.5, y2);
-
-	cairo_destroy(cr);
 }
 
 void
 hc_draw_expander (GtkStyle        *style,
-                           GdkWindow       *window,
+                           cairo_t         *cr,
                            GtkStateType     state_type,
-                           GdkRectangle    *area,
                            GtkWidget       *widget,
                            const gchar     *detail,
                            gint             x,
@@ -1072,12 +989,9 @@ hc_draw_expander (GtkStyle        *style,
   double x_double_vert, y_double_vert;
   double x_double, y_double;
   gint degrees = 0;
-  cairo_t *cr;
 	
   CHECK_ARGS
 
-  cr = ge_gdk_drawable_to_cairo (window, area);
-
   if (widget &&
       gtk_widget_class_find_style_property (GTK_WIDGET_GET_CLASS (widget),
 					    "expander-size"))
@@ -1168,16 +1082,13 @@ hc_draw_expander (GtkStyle        *style,
   
   ge_cairo_set_color (cr, &hc_style->color_cube.text[state_type]);
   cairo_stroke (cr);
-  
-  cairo_destroy (cr);
 }
 
 void
 hc_draw_diamond (GtkStyle      *style,
-		 GdkWindow     *window,
+		 cairo_t       *cr,
 		 GtkStateType   state_type,
 		 GtkShadowType  shadow_type,
-		 GdkRectangle  *area,
 		 GtkWidget     *widget,
 		 const gchar   *detail,
 		 gint           x,
@@ -1189,16 +1100,12 @@ hc_draw_diamond (GtkStyle      *style,
 
 	int half_width;
 	int half_height;
-	cairo_t *cr;
 	
 	CHECK_ARGS
-	SANITIZE_SIZE
 	
 	half_width = width / 2;
 	half_height = height / 2;
 
-	cr = ge_gdk_drawable_to_cairo (window, area);
-
 	switch (shadow_type) {
 	case GTK_SHADOW_IN:
 		ge_cairo_line(cr, &hc_style->color_cube.light[state_type],
@@ -1281,123 +1188,4 @@ hc_draw_diamond (GtkStyle      *style,
 	default:
 		break;
 	}
-
-	cairo_destroy(cr);
-}
-
-void
-hc_draw_polygon (GtkStyle      *style,
-		 GdkWindow     *window,
-		 GtkStateType   state_type,
-		 GtkShadowType  shadow_type,
-		 GdkRectangle  *area,
-		 GtkWidget     *widget,
-		 const gchar   *detail,
-		 GdkPoint      *points,
-		 gint           npoints,
-		 gboolean       fill)
-{
-	HcStyle *hc_style = HC_STYLE (style);
-
-	static const gdouble pi_over_4 = G_PI_4;
-	static const gdouble pi_3_over_4 = G_PI_4 * 3;
-	
-	CairoColor           *color1;
-	CairoColor           *color2;
-	CairoColor           *color3;
-	CairoColor           *color4;
-	gdouble            angle;
-	int                xadjust;
-	int                yadjust;
-	int                i;
-	cairo_t           *cr;
-
-	g_return_if_fail(style != NULL);
-	g_return_if_fail(window != NULL);
-	g_return_if_fail(points != NULL);
-	
-	switch (shadow_type) {
-	case GTK_SHADOW_IN:
-		color1 = &hc_style->color_cube.light[state_type];
-		color2 = &hc_style->color_cube.dark[state_type];
-		color3 = &hc_style->color_cube.light[state_type];
-		color4 = &hc_style->color_cube.dark[state_type];
-		break;
-	case GTK_SHADOW_ETCHED_IN:
-		color1 = &hc_style->color_cube.light[state_type];
-		color2 = &hc_style->color_cube.dark[state_type];
-		color3 = &hc_style->color_cube.dark[state_type];
-		color4 = &hc_style->color_cube.light[state_type];
-		break;
-	case GTK_SHADOW_OUT:
-		color1 = &hc_style->color_cube.dark[state_type];
-		color2 = &hc_style->color_cube.light[state_type];
-		color3 = &hc_style->color_cube.dark[state_type];
-		color4 = &hc_style->color_cube.light[state_type];
-		break;
-	case GTK_SHADOW_ETCHED_OUT:
-		color1 = &hc_style->color_cube.dark[state_type];
-		color2 = &hc_style->color_cube.light[state_type];
-		color3 = &hc_style->color_cube.light[state_type];
-		color4 = &hc_style->color_cube.dark[state_type];
-		break;
-	default:
-		return;
-	}
-
-	cr = ge_gdk_drawable_to_cairo (window, area);
-	
-	if (fill)
-		ge_cairo_polygon(cr, &hc_style->color_cube.bg[state_type], points, npoints);
-	
-	npoints--;
-	
-	for (i = 0; i < npoints; i++) {
-		if ((points[i].x == points[i + 1].x) &&
-		    (points[i].y == points[i + 1].y)) {
-			angle = 0;
-		} else {
-			angle = atan2(points[i + 1].y - points[i].y,
-				      points[i + 1].x - points[i].x);
-		}
-		
-		if ((angle > -pi_3_over_4) && (angle < pi_over_4)) {
-			if (angle > -pi_over_4) {
-				xadjust = 0;
-				yadjust = 1;
-			} else {
-				xadjust = 1;
-				yadjust = 0;
-			}
-
-			ge_cairo_line(cr, color1,
-				      points[i].x - xadjust, 
-				      points[i].y - yadjust,
-				      points[i + 1].x - xadjust, 
-				      points[i + 1].y - yadjust);
-			ge_cairo_line(cr, color3,
-				      points[i].x, points[i].y,
-				      points[i + 1].x, points[i + 1].y);
-		}
-		else {
-			if ((angle < -pi_3_over_4) || (angle > pi_3_over_4)) {
-				xadjust = 0;
-				yadjust = 1;
-			} else {
-				xadjust = 1;
-				yadjust = 0;
-			}
-			
-			ge_cairo_line(cr, color4,
-				      points[i].x + xadjust, 
-				      points[i].y + yadjust,
-				      points[i + 1].x + xadjust, 
-				      points[i + 1].y + yadjust);
-			ge_cairo_line(cr, color2,
-				      points[i].x, points[i].y,
-				      points[i + 1].x, points[i + 1].y);
-		}
-	}
-
-	cairo_destroy(cr);
 }
