@@ -1,3 +1,34 @@
+/* Helper functions for gtk-engines
+ *
+ * Copyright (C) 2006 Andrew Johnson <acjgenius@earthlink.net>
+ * Copyright (C) 2006 Thomas Wood <thos@gnome.org>
+ * Copyright (C) 2006-2007 Benjamin Berg <benjamin@sipsolutions.net>
+ * Copyright (C) 2007 Andrea Cimitan <andrea.cimitan@gmail.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Project contact: <gnome-themes-list@gnome.org>
+ *
+ *
+ * Written by Andrew Johnson <acjgenius@earthlink.net>
+ * Written by Benjamin Berg <benjamin@sipsolutions.net>
+ * Written by Thomas Wood <thos@gnome.org>
+ * modified by Andrea Cimitan <andrea.cimitan@gmail.com>
+ *
+ */
+
 #include <math.h>
 #include "general-support.h"
 #include "cairo-support.h"
@@ -81,31 +112,31 @@ ge_color_from_hsb (gdouble hue,
 	gdouble m1, m2, m3;
 
 	if (!color) return;
-  	  
+
 	if (brightness <= 0.5)
 		m2 = brightness * (1 + saturation);
 	else
 		m2 = brightness + saturation - brightness * saturation;
- 
+
 	m1 = 2 * brightness - m2;
- 
+
 	hue_shift[0] = hue + 120;
 	hue_shift[1] = hue;
 	hue_shift[2] = hue - 120;
- 
+
 	color_shift[0] = color_shift[1] = color_shift[2] = brightness;	
- 
+
 	i = (saturation == 0)?3:0;
- 
+
 	for (; i < 3; i++)
 	{
 		m3 = hue_shift[i];
- 
+
 		if (m3 > 360)
 			m3 = MODULA(m3, 360);
 		else if (m3 < 0)
 			m3 = 360 - MODULA(ABS(m3), 360);
- 
+
 		if (m3 < 60)
 			color_shift[i] = m1 + (m2 - m1) * m3 / 60;
 		else if (m3 < 180)
@@ -114,8 +145,8 @@ ge_color_from_hsb (gdouble hue,
 			color_shift[i] = m1 + (m2 - m1) * (240 - m3) / 60;
 		else
 			color_shift[i] = m1;
-	}	
- 
+	}
+
 	color->r = color_shift[0];
 	color->g = color_shift[1];
 	color->b = color_shift[2];	
@@ -163,7 +194,7 @@ ge_gtk_style_to_cairo_color_cube (GtkStyle * style, CairoColorCube *cube)
 	g_return_if_fail (style && cube);
 
 	for (i = 0; i < 5; i++)
-	{ 
+	{
 		ge_gdk_color_to_cairo (&style->bg[i], &cube->bg[i]);
 		ge_gdk_color_to_cairo (&style->fg[i], &cube->fg[i]);
 
@@ -174,7 +205,7 @@ ge_gtk_style_to_cairo_color_cube (GtkStyle * style, CairoColorCube *cube)
 		ge_gdk_color_to_cairo (&style->base[i], &cube->base[i]);
 		ge_gdk_color_to_cairo (&style->text[i], &cube->text[i]);
 		ge_gdk_color_to_cairo (&style->text_aa[i], &cube->text_aa[i]);
-    	}
+	}
 
 	cube->black.r = cube->black.g = cube->black.b = 0;
 	cube->black.a = 1;
@@ -189,17 +220,27 @@ ge_shade_color(const CairoColor *base, gdouble shade_ratio, CairoColor *composit
 	gdouble hue = 0;
 	gdouble saturation = 0;
 	gdouble brightness = 0;
- 
+
 	g_return_if_fail (base && composite);
+	
+	if (shade_ratio == 1.0)
+	{
+		composite->r = base->r;
+		composite->g = base->g;
+		composite->b = base->b;
+		composite->a = base->a;
+		
+		return;
+	}
 
 	ge_hsb_from_color (base, &hue, &saturation, &brightness);
- 
+
 	brightness = MIN(brightness*shade_ratio, 1.0);
 	brightness = MAX(brightness, 0.0);
-  
+
 	saturation = MIN(saturation*shade_ratio, 1.0);
 	saturation = MAX(saturation, 0.0);
-  
+
 	ge_color_from_hsb (hue, saturation, brightness, composite);
 	composite->a = base->a;	
 }
@@ -210,7 +251,7 @@ ge_saturate_color (const CairoColor *base, gdouble saturate_level, CairoColor *c
 	gdouble hue = 0;
 	gdouble saturation = 0;
 	gdouble brightness = 0;
- 
+
 	g_return_if_fail (base && composite);
 
 	ge_hsb_from_color (base, &hue, &saturation, &brightness);
@@ -223,7 +264,7 @@ ge_saturate_color (const CairoColor *base, gdouble saturate_level, CairoColor *c
 }
 
 void
-ge_mix_color (const CairoColor *color1, const CairoColor *color2, 
+ge_mix_color (const CairoColor *color1, const CairoColor *color2,
               gdouble mix_factor, CairoColor *composite)
 {
 	g_return_if_fail (color1 && color2 && composite);
@@ -246,7 +287,7 @@ ge_gdk_drawable_to_cairo (GdkDrawable  *window, GdkRectangle *area)
 	cairo_set_line_cap (cr, CAIRO_LINE_CAP_SQUARE);
 	cairo_set_line_join (cr, CAIRO_LINE_JOIN_MITER);
 
-	if (area) 
+	if (area)
 	{
 		cairo_rectangle (cr, area->x, area->y, area->width, area->height);
 		cairo_clip_preserve (cr);
@@ -277,8 +318,8 @@ ge_cairo_set_gdk_color_with_alpha (cairo_t *cr, const GdkColor *color, gdouble a
 
 void 
 ge_cairo_pattern_add_color_stop_color (cairo_pattern_t *pattern, 
-						gfloat offset, 
-						const CairoColor *color)
+                                       gfloat offset, 
+                                       const CairoColor *color)
 {
 	g_return_if_fail (pattern && color);
 
@@ -286,10 +327,10 @@ ge_cairo_pattern_add_color_stop_color (cairo_pattern_t *pattern,
 }
 
 void
-ge_cairo_pattern_add_color_stop_shade(cairo_pattern_t *pattern, 
-						gdouble offset, 
-						const CairoColor *color, 
-						gdouble shade)
+ge_cairo_pattern_add_color_stop_shade (cairo_pattern_t *pattern, 
+                                       gdouble offset, 
+                                       const CairoColor *color, 
+                                       gdouble shade)
 {
 	CairoColor shaded;
 
@@ -302,12 +343,14 @@ ge_cairo_pattern_add_color_stop_shade(cairo_pattern_t *pattern,
 		ge_shade_color(color, shade, &shaded);
 	}
 
-	ge_cairo_pattern_add_color_stop_color(pattern, offset, &shaded);	
+	ge_cairo_pattern_add_color_stop_color(pattern, offset, &shaded);
 }
 
-/* This function will draw a rounded corner at position x,y. If the radius
+/* 
+ * This function will draw a rounded corner at position x,y. If the radius
  * is very small (or negative) it will instead just do a line_to.
- * ge_cairo_rounded_corner assumes clockwise drawing. */
+ * ge_cairo_rounded_corner assumes clockwise drawing.
+ */
 void
 ge_cairo_rounded_corner (cairo_t      *cr,
                          double        x,
@@ -321,28 +364,29 @@ ge_cairo_rounded_corner (cairo_t      *cr,
 	}
 	else
 	{
-		switch (corner) {
-		case CR_CORNER_NONE:
-			cairo_line_to (cr, x, y);
-			break;
-		case CR_CORNER_TOPLEFT:
-			cairo_arc (cr, x + radius, y + radius, radius, G_PI, G_PI * 3/2);
-			break;
-		case CR_CORNER_TOPRIGHT:
-			cairo_arc (cr, x - radius, y + radius, radius, G_PI * 3/2, G_PI * 2);
-			break;
-		case CR_CORNER_BOTTOMRIGHT:
-			cairo_arc (cr, x - radius, y - radius, radius, 0, G_PI * 1/2);
-			break;
-		case CR_CORNER_BOTTOMLEFT:
-			cairo_arc (cr, x + radius, y - radius, radius, G_PI * 1/2, G_PI);
-			break;
+		switch (corner)
+		{
+			case CR_CORNER_NONE:
+				cairo_line_to (cr, x, y);
+				break;
+			case CR_CORNER_TOPLEFT:
+				cairo_arc (cr, x + radius, y + radius, radius, G_PI, G_PI * 3/2);
+				break;
+			case CR_CORNER_TOPRIGHT:
+				cairo_arc (cr, x - radius, y + radius, radius, G_PI * 3/2, G_PI * 2);
+				break;
+			case CR_CORNER_BOTTOMRIGHT:
+				cairo_arc (cr, x - radius, y - radius, radius, 0, G_PI * 1/2);
+				break;
+			case CR_CORNER_BOTTOMLEFT:
+				cairo_arc (cr, x + radius, y - radius, radius, G_PI * 1/2, G_PI);
+				break;
 
-		default:
-			/* A bitfield and not a sane value ... */
-			g_assert_not_reached ();
-			cairo_line_to (cr, x, y);
-			return;
+			default:
+				/* A bitfield and not a sane value ... */
+				g_assert_not_reached ();
+				cairo_line_to (cr, x, y);
+				return;
 		}
 	}
 }
@@ -395,9 +439,9 @@ ge_cairo_rounded_rectangle (cairo_t *cr,
 
 /* ge_cairo_stroke_rectangle.
  *
- *  A simple function to stroke the rectangle { x, y, w, h}.
- *  (This function only exists because of a cairo performance bug that
- *    has been fixed and it may be a good idea to get rid of it again.)
+ *   A simple function to stroke the rectangle { x, y, w, h}.
+ *   (This function only exists because of a cairo performance bug that
+ *   has been fixed and it may be a good idea to get rid of it again.)
  */
 void
 ge_cairo_stroke_rectangle (cairo_t *cr, double x, double y, double w, double h)
@@ -406,19 +450,48 @@ ge_cairo_stroke_rectangle (cairo_t *cr, double x, double y, double w, double h)
 	cairo_stroke (cr);
 }
 
+void
+ge_cairo_inner_rectangle (cairo_t *cr,
+                          double x, double y,
+                          double width, double height)
+{
+	double line_width = cairo_get_line_width (cr);
+
+	cairo_rectangle (cr, x + line_width / 2.0,
+	                     y + line_width / 2.0,
+	                     width - line_width,
+	                     height - line_width);
+}
+
+void
+ge_cairo_inner_rounded_rectangle (cairo_t *cr,
+                                  double x, double y,
+                                  double width, double height,
+                                  double radius, CairoCorners corners)
+{
+	double line_width = cairo_get_line_width (cr);
+
+	ge_cairo_rounded_rectangle (cr,
+                                    x + line_width / 2.0,
+                                    y + line_width / 2.0,
+                                    width - line_width,
+                                    height - line_width,
+                                    radius, corners);
+}
+
 /***********************************************
  * ge_cairo_simple_border -
  *  
  *   A simple routine to draw thin squared
  *   borders with a topleft and bottomright color.
- *    
+ *   
  *   It originated in Smooth-Engine.
  ***********************************************/
 void
 ge_cairo_simple_border (cairo_t *cr,
-				const CairoColor * tl, const CairoColor * br,
-				gint x,	gint y, gint width, gint height, 
-				gboolean topleft_overlap)
+                        const CairoColor * tl, const CairoColor * br,
+                        gint x,	gint y, gint width, gint height, 
+                        gboolean topleft_overlap)
 {
 	gboolean solid_color;
 
@@ -445,7 +518,7 @@ ge_cairo_simple_border (cairo_t *cr,
 		
 		cairo_stroke (cr);
 	}
- 
+
 	ge_cairo_set_color(cr, tl);	
 
 	cairo_move_to(cr, x + 0.5, y + height - 0.5);
@@ -471,9 +544,9 @@ ge_cairo_simple_border (cairo_t *cr,
 }
 
 void ge_cairo_polygon (cairo_t *cr,
-				const CairoColor *color,
-				GdkPoint *points,
-				gint npoints)
+                       const CairoColor *color,
+                       GdkPoint *points,
+                       gint npoints)
 {
 	int i = 0;
 
@@ -503,12 +576,12 @@ void ge_cairo_polygon (cairo_t *cr,
 }
 
 void ge_cairo_line (cairo_t *cr,
-			const CairoColor *color,
-			gint x1,
-			gint y1,
-			gint x2,
-			gint y2)
-{ 
+                    const CairoColor *color,
+                    gint x1,
+                    gint y1,
+                    gint x2,
+                    gint y2)
+{
 	cairo_save(cr);
 
 	ge_cairo_set_color(cr, color);	
@@ -584,18 +657,19 @@ ge_cairo_exchange_axis (cairo_t  *cr,
  ***********************************************/
 void 
 ge_cairo_pattern_fill(cairo_t *canvas,
-			CairoPattern *pattern,
-			gint x,
-			gint y,
-			gint width,
-			gint height)
+                      CairoPattern *pattern,
+                      gint x,
+                      gint y,
+                      gint width,
+                      gint height)
 {
 	cairo_matrix_t original_matrix, current_matrix;
 
 	if (pattern->operator == CAIRO_OPERATOR_DEST)
-	{
 		return;
-	}
+	
+	if (width <= 0 || height <= 0)
+		return;
 
 	cairo_pattern_get_matrix(pattern->handle, &original_matrix);
 	current_matrix = original_matrix;
@@ -641,7 +715,7 @@ ge_cairo_pattern_fill(cairo_t *canvas,
 	cairo_save(canvas);
 
 	cairo_set_source(canvas, pattern->handle);
-        cairo_set_operator(canvas, pattern->operator);
+	cairo_set_operator(canvas, pattern->operator);
 	cairo_rectangle(canvas, x, y, width, height);
 
 	cairo_fill (canvas);
@@ -658,7 +732,7 @@ ge_cairo_pattern_fill(cairo_t *canvas,
  ***********************************************/
 CairoPattern*
 ge_cairo_color_pattern(CairoColor *base)
-{	
+{
 	CairoPattern * result = g_new0(CairoPattern, 1);
 
 	#if  ((CAIRO_VERSION_MAJOR < 1) || ((CAIRO_VERSION_MAJOR == 1) && (CAIRO_VERSION_MINOR < 2)))
@@ -668,10 +742,10 @@ ge_cairo_color_pattern(CairoColor *base)
 	result->scale = GE_DIRECTION_NONE;
 	result->translate = GE_DIRECTION_NONE;
 
-	result->handle = cairo_pattern_create_rgba(base->r, 
-							base->g, 
-							base->b, 
-							base->a);
+	result->handle = cairo_pattern_create_rgba(base->r,
+	                                           base->g,
+	                                           base->b, 
+	                                           base->a);
 
 	result->operator = CAIRO_OPERATOR_SOURCE;
 	
@@ -685,20 +759,14 @@ ge_cairo_color_pattern(CairoColor *base)
  ***********************************************/
 CairoPattern*
 ge_cairo_pixbuf_pattern(GdkPixbuf *pixbuf)
-{	
-	CairoPattern * result = g_new0(CairoPattern, 1);
+{
+	CairoPattern * result;
+        cairo_pattern_t *pattern;
 
 	cairo_t *canvas;
 	cairo_surface_t * surface;
 	gint width, height;
 
-	#if  ((CAIRO_VERSION_MAJOR < 1) || ((CAIRO_VERSION_MAJOR == 1) && (CAIRO_VERSION_MINOR < 2)))
-		result->type = CAIRO_PATTERN_TYPE_SURFACE;
-	#endif
-
-	result->scale = GE_DIRECTION_NONE;
-	result->translate = GE_DIRECTION_BOTH;
-
 	width = gdk_pixbuf_get_width(pixbuf);
 	height = gdk_pixbuf_get_height(pixbuf);
 	
@@ -711,38 +779,30 @@ ge_cairo_pixbuf_pattern(GdkPixbuf *pixbuf)
 	cairo_fill (canvas);
 	cairo_destroy(canvas);
 
-	result->handle = cairo_pattern_create_for_surface (surface);
+	pattern = cairo_pattern_create_for_surface (surface);
 	cairo_surface_destroy(surface);
 
-	cairo_pattern_set_extend (result->handle, CAIRO_EXTEND_REPEAT);
+	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
 
-	result->operator = CAIRO_OPERATOR_SOURCE;
+        result = ge_cairo_pattern_pattern (pattern);
+        cairo_pattern_destroy (pattern);
 
 	return result;
 }
 
-/***********************************************
- * ge_cairo_pixmap_pattern -
- *  
- *   Create A Tiled Pixmap Pattern
- ***********************************************/
 CairoPattern*
-ge_cairo_pixmap_pattern(GdkPixmap *pixmap)
-{	
-	CairoPattern * result = NULL;
+ge_cairo_pattern_pattern(cairo_pattern_t *pattern)
+{
+	CairoPattern * result = g_new0(CairoPattern, 1);
 
-	GdkPixbuf * pixbuf;
-	gint width, height;
+	#if ((CAIRO_VERSION_MAJOR < 1) || ((CAIRO_VERSION_MAJOR == 1) && (CAIRO_VERSION_MINOR < 2)))
+		result->type = CAIRO_PATTERN_TYPE_SURFACE;
+	#endif
 
-	gdk_drawable_get_size (GDK_DRAWABLE (pixmap), &width, &height);
-
-	pixbuf = gdk_pixbuf_get_from_drawable(NULL, GDK_DRAWABLE (pixmap), 
-				gdk_drawable_get_colormap(GDK_DRAWABLE (pixmap)), 
-				0, 0, 0, 0, width, height);
-
-	result = ge_cairo_pixbuf_pattern(pixbuf);
-	
-	g_object_unref (pixbuf);
+	result->scale = GE_DIRECTION_NONE;
+	result->translate = GE_DIRECTION_BOTH;
+        result->handle = cairo_pattern_reference (pattern);
+	result->operator = CAIRO_OPERATOR_SOURCE;
 
 	return result;
 }
@@ -757,9 +817,9 @@ ge_cairo_pixmap_pattern(GdkPixmap *pixmap)
  ***********************************************/
 CairoPattern *
 ge_cairo_linear_shade_gradient_pattern(CairoColor *base, 
-						gdouble shade1, 
-						gdouble shade2, 
-						gboolean vertical)
+                                       gdouble shade1, 
+                                       gdouble shade2, 
+                                       gboolean vertical)
 {
 	CairoPattern * result = g_new0(CairoPattern, 1);
 	
@@ -796,12 +856,45 @@ ge_cairo_pattern_destroy(CairoPattern *pattern)
 	{
 		if (pattern->handle)
 			cairo_pattern_destroy(pattern->handle);
-			
+		
 		g_free(pattern);
 	}
 }
 
-/* The following function will be called by GTK+ when the module
+void
+ge_cairo_transform_for_layout (cairo_t *cr,
+                               PangoLayout *layout,
+                               int x,
+                               int y)
+{
+        const PangoMatrix *matrix;
+
+        matrix = pango_context_get_matrix (pango_layout_get_context (layout));
+        if (matrix)
+        {
+                cairo_matrix_t cairo_matrix;
+                PangoRectangle rect;
+                
+                cairo_matrix_init (&cairo_matrix,
+                                   matrix->xx, matrix->yx,
+                                   matrix->xy, matrix->yy,
+                                   matrix->x0, matrix->y0);
+
+                pango_layout_get_extents (layout, NULL, &rect);
+                pango_matrix_transform_rectangle (matrix, &rect);
+                pango_extents_to_pixels (&rect, NULL);
+                                                    
+                cairo_matrix.x0 += x - rect.x;
+                cairo_matrix.y0 += y - rect.y;
+
+                cairo_set_matrix (cr, &cairo_matrix);
+        }
+        else
+                cairo_translate (cr, x, y);
+}
+
+/* 
+ * The following function will be called by GTK+ when the module
  * is loaded and checks to see if we are compatible with the
  * version of GTK+ that loads us.
  */
@@ -809,7 +902,7 @@ GE_EXPORT const gchar* g_module_check_init (GModule *module);
 const gchar*
 g_module_check_init (GModule *module)
 {
-  return gtk_check_version (GTK_MAJOR_VERSION,
-			    GTK_MINOR_VERSION,
-			    GTK_MICRO_VERSION - GTK_INTERFACE_AGE);
+	return gtk_check_version (GTK_MAJOR_VERSION,
+	                          GTK_MINOR_VERSION,
+	                          GTK_MICRO_VERSION - GTK_INTERFACE_AGE);
 }
